"""A generic file to manipulate graph description files (GDL)

(Put some longer documentation here in the future)
"""
import vcg_GraphNode
import vcg_GraphLink
import vcg_parse
import string


class vcgGraph:
  """A class for manipulating .vcg Graph files (as used by Wingraph32 or AiSee)
  
  Some longer documentation will be put here in the future
  
  """    
  def __init__(self, attributes = {"title":'"Graph"', "manhattan_edges":"yes", "layoutalgorithm":"mindepth"}):
    self.nodes = {}
    self.links = []
    self.attributes = {}
    for k, v in attributes.items():
      self.attributes[ k ] = v
  
  def add_attributes( attributes ):
    for k, v in attributes.items():
      self.attributes[ k ] = v

  def set_attribute( self, attrib_name, attrib_string ):
    self.attributes[ attrib_name ] = attrib_string
    
  def get_attribute( self, attrib_name ):
    return self.attributes[ attrib_name ]

  def gen_VCG_string(self):
    ida_pallete = "// IDA palette\ncolorentry 32: 0 0 0\ncolorentry 33: 0 0 255\ncolorentry 34: 0 0 255\ncolorentry 35: 128 128 128\ncolorentry 36: 128 128 128\ncolorentry 37: 0 0 128\ncolorentry 38: 0 0 128\ncolorentry 39: 0 0 255\ncolorentry 40: 0 0 255\ncolorentry 41: 0 0 128\ncolorentry 42: 0 128 0\ncolorentry 43: 0 255 0\ncolorentry 44: 0 128 0\ncolorentry 45: 255 128 0\ncolorentry 46: 0 128 0\ncolorentry 47: 128 128 255\ncolorentry 48: 255 0 0\ncolorentry 49: 128 128 0\ncolorentry 50: 1 1 1\ncolorentry 51: 192 192 192\ncolorentry 52: 0 0 255\ncolorentry 53: 0 0 255\ncolorentry 54: 0 0 255\ncolorentry 55: 128 128 128\ncolorentry 56: 128 128 255\ncolorentry 57: 0 128 0\ncolorentry 58: 0 0 128\ncolorentry 59: 0 0 255\ncolorentry 60: 128 0 128\ncolorentry 61: 0 128 0\ncolorentry 62: 0 128 0\ncolorentry 63: 0 128 64\ncolorentry 64: 0 0 128\ncolorentry 65: 0 0 128\ncolorentry 66: 255 0 255\ncolorentry 67: 128 128 0\ncolorentry 68: 0 0 128\ncolorentry 69: 0 0 255\ncolorentry 70: 0 0 128\ncolorentry 71: 0 0 255\ncolorentry 72: 0 0 0\ncolorentry 73: 255 255 255\ncolorentry 74: 192 192 192\ncolorentry 75: 0 255 255\ncolorentry 76: 0 0 0\ncolorentry 77: 128 0 0\ncolorentry 78: 128 128 128\ncolorentry 79: 128 128 0\ncolorentry 80: 255 0 255\ncolorentry 81: 0 0 0\ncolorentry 82: 0 0 255\ncolorentry 83: 0 0 0\n"
    output = "graph: { \n\t"
    for k, v in self.attributes.items():
      output = output + k + ": " + v + "\n\t"
    output += ida_pallete;
    for k, v in self.nodes.items():
      output = output + v.make_vcg_output()
    for k in self.links:
      output = output + k.make_vcg_output()
    output = output + "}\n"
    return output
    
  def write_VCG_File(self, filename):
    f = open( filename, 'w' )
    s = self.gen_VCG_string()
    f.write( s )
    f.close()
    
  def load_VCG_File(self, filename):
    file = open( filename, "r" )
    completestring = file.read()        # read the entire file
    # now start iterating through the file, first getting all nodes
    in_node = vcg_parse.vcg_get_item( completestring, "node" )
    while in_node != 0:
      nodetitle = vcg_parse.vcg_get_enclosed_attribute( in_node, "title" )
      # print "Adding node with title: "+nodetitle[1:-1]
      newnode = self.Add_Node( nodetitle[1:-1] ) 
      newnode.parse_vcg_output( in_node )
      # cut the node away ... 
      chopstring = string.replace( completestring, in_node, "" )
      completestring = chopstring
      # get next one
      in_node = vcg_parse.vcg_get_item( completestring, "node" )
    in_link = vcg_parse.vcg_get_item( completestring, "edge" )    
    while in_link != 0:
      linksource = vcg_parse.vcg_get_enclosed_attribute( in_link, "sourcename" )
      linktarget = vcg_parse.vcg_get_enclosed_attribute( in_link, "targetname" )
      color = vcg_parse.vcg_get_enclosed_attribute( in_link, "color")
      newlink = self.Add_Link( linksource[1:-1], linktarget[1:-1] )
      newlink.parse_vcg_output(in_link)
      # cut the link away
      chopstring = string.replace( completestring, in_link, "" )
      srchidx = string.find( chopstring, in_link )
      completestring = chopstring
      # get next one
      in_link = vcg_parse.vcg_get_item( completestring, "edge" )
      
  def Add_Node(self, nodename):
    self.nodes[ nodename ] = vcg_GraphNode.vcgGraphNode( nodename )
    return self.nodes[ nodename ]
  
  def Add_Link(self, sourcename, targetname):
    link = vcg_GraphLink.vcgGraphLink( sourcename, targetname )
    self.links.append( link )
    return link
    
  def Del_Node(self, nodename):
    # Create a temporary copy of the list
    removelinks = []
    for x in self.links:                        # remove the links to/from 
      if x.targetname == nodename:
        removelinks.append( x )
      if x.sourcename == nodename:              # this node
        removelinks.append( x )
    for x in removelinks:
      if x in self.links:
        self.links.remove( x )
    del self.nodes[nodename]
    return

  def Del_Link(self, sourcename, targetname):
    for x in self.links:                        # remove the relevant link
      if x.sourcename == sourcename:              
        if x.targetname == targetname:
          self.links.remove( x )
    
  def Get_Downlinks_To( self, nodename ):
    linkslist = []
    for x in self.links:
      if x.targetname == nodename:
        linkslist.append( x )
    return linkslist
    
  def Get_Downlinks_From( self, nodename ):
    linkslist = []
    for x in self.links:
      if x.sourcename == nodename:
        linkslist.append( x )
    return linkslist

  def Get_Parents( self, nodename ):
    parentnames = []
    linkslist = self.Get_Downlinks_To( nodename )
    for x in linkslist:
      parentnames.append( x.sourcename )
    return parentnames
    
  def Get_Children( self, nodename ):
    childnames = []
    linkslist = self.Get_Downlinks_From( nodename )
    for x in linkslist:
      childnames.append( x.targetname )
    return childnames
    
  def Get_Node( self, nodename ):
    is_valid_name = 0
    for x in self.nodes.keys():
      if x == nodename:
        is_valid_name = 1
    if is_valid_name:
      return self.nodes[ nodename ]
    return 0

  def Get_Top_Nodes( self ):
    topnodes = []
    for x in self.nodes.keys():
      bLinkedTo = 0
      for y in self.links:
        if( x == y.targetname ):
          bLinkedTo = 1
          break
      if( bLinkedTo == 0 ):
        topnodes.append( x )
    return topnodes
    
  def Get_Bottom_Nodes( self ):
    bottomnode = []
    for x in self.nodes.keys():
      bLinkedFrom = 0
      for y in self.links:
        if( x == y.sourcename ):
          bLinkedFrom = 1
          break
      if( bLinkedFrom == 0 ):
        bottomnodes.append( x )
    return bottomnodes
  
  def Get_Nodes_Before( self, node ):
    for x in self.nodes.keys():
      self.nodes[ x ].visited = 0
    beforeset = []
    workset = []
    workset.append( node )
    print "Len(workset) is %d" % len( workset )
    while len( workset ) != 0:
      print beforeset
      nextnode = workset[ 0 ]
      workset = workset[1:]
      parentset = self.Get_Parents( nextnode )
      print "PARENTSET:"
      print parentset
      print "----------"
      for x in parentset:
        if self.nodes[ x ].visited == 0:
          self.nodes[ x ].visited = 1
          beforeset.append( x )
          workset.append( x )
    print beforeset
    return beforeset
    
  def Get_Nodes_After( self, node ):
    for x in self.nodes.keys():
      self.nodes[ x ].visited = 0
    beforeset = []
    workset = []
    workset.append( node )
#    print "Len(workset) is %d" % len( workset )
    while len( workset ) != 0:
#      print "Beforeset: "
      print beforeset
      nextnode = workset[ 0 ]
      workset = workset[1:]
#      print "Getting children of %s" % nextnode
      parentset = self.Get_Children( nextnode )
#      print "Childrenset is:"
#      print parentset
      for x in parentset:
        if self.nodes[ x ].visited == 0:
          self.nodes[ x ].visited = 1
          beforeset.append( x )
          workset.append( x )
#    print beforeset
    return beforeset
    
  def Get_Path_From_To( self, nodeBegin, nodeEnd ):
    preset = self.Get_Nodes_Before( nodeEnd )
    postset = self.Get_Nodes_After( nodeBegin )
#    print "Preset is: " 
#    print preset
#    print "Postset is: " 
#    print postset
    intersect = []
    for x in preset:
      for y in postset:
        if x == y:
          intersect.append( x )
    return intersect
    
  def make_daVinci_output( self ):
    davinci_output = '['
    for key, node in self.nodes.items():
      davinci_output += node.make_daVinci_output( self ) 
      davinci_output += ','
    return davinci_output[:-1] + ']'

  def make_GML_output( self ):
    output = 'Creator: "vcgGraph.py"\nVersion 2.2\ngraph\n['
    output += "\thierarchic 1\n"
    output += "\tlabel "+self.attributes[ "title" ]+"\n"
    for key, node in self.nodes.items():
      output += node.make_GML_output( self )
    for link in self.links:
      output += link.make_GML_output()
    output += "]\n"
    return output
