import string
import vcg_parse

class vcgGraphLink:
  """A class that represents the internal Graphlink used for dealing with .vcg's
  
  Some longer documentation will be put here in the future
  
  """      
  def __init__(self, source, target, attributes = { 'color':'black' }):
    self.sourcename = source
    self.targetname = target
    self.attributes = {}
    for k, v in attributes.items():
      self.attributes[ k ] = v
    
  def set_attribute( self, attrib_name, attrib_string ):
    self.attributes[ attrib_name ] = attrib_string
    
  def get_attribute( self, attrib_name ):
    return self.attributes[ attrib_name ]
    
  def add_attributes( self, attributes ):
    for k, v in attributes.items():
      self.attributes[ k ] = v
    
  def make_vcg_output( self ):
    output = 'edge: { sourcename: "' + self.sourcename + '" targetname: "' + self.targetname +'" \n\t'
    for k, v in self.attributes.items():
      output = output + k + ": " + v + "\n\t"
    output = output + '}\n'
    return output
    
  def parse_vcg_output( self, vcg_string ):
      # skip targetname and sourcename attributes

      tokens = vcg_string.split()
      for t in range(len(tokens)):
          token = tokens[t]
          if token[-1] == ':':
              # its an attribute
              if token not in ['edge:','targetname:','sourcename:']:
                  self.set_attribute(token[:-1], tokens[t+1])

  def make_daVinci_output( self ):
    Unique_ID = self.sourcename+"_"+self.targetname
    davinci_string = 'l("edge_' + Unique_ID + '", e("",[], r("node_'+ self.targetname +'")))'
    return davinci_string

  HTML_colors = {
    'red': '#ff0000',
    'green': '#00ff00',
    'darkgreen': '#008000',
    'blue' : '#0000ff',
    'black': '#000000',
  }
  
  def get_HTML_color(self):
    try:
      return self.HTML_colors[self.get_attribute('color')]
    except:
      return '#ffff00'
    
  def make_GML_output( self ):
    GMLstring = '\tedge\n\t[\n\t\tsource "' + self.sourcename + '"\n'
    GMLstring += '\t\ttarget "' + self.targetname + '"\n'
    GMLstring += '\t\tgraphics\n\t\t[\n\t\t\tfill "%s"\n\t\t\ttargetArrow "standard"\n\t\t]\n\t]\n' % self.get_HTML_color()
    return GMLstring
